//==============================================
//      File rutils.pas
//
//      Components library for Delphi and BCB++.
//      ,   .
//      Procedures, functions and types.
//
//      Copyright 1998-2000 Polaris Software
//      http://members.xoom.com/PolarisSoft
//      mailto: PolarisLib@mail.ru
//==============================================
unit rUtils;

{$I POLARIS.INC}

interface

uses Classes, Windows, SysUtils, Graphics, Consts, Registry, Controls, StrUtils
     {$IFDEF GIF}, RxGIF{$ENDIF}{$IFDEF JPEG}, JPEG{$ENDIF};

type
  TrDate = record
             D,M,Y: Word;
           end;

  TBitmapResKind =
    (rkNone, rkBitmap, rkIcon, rkCursor, rkEMF, rkWMF
     {$IFDEF GIF}, rkGIF{$ENDIF}{$IFDEF JPEG}, rkJPEG{$ENDIF});

const
  DefBRK = rkBitmap;
  ResKindNames: array[TBitmapResKind] of PChar =
    ('',RT_BITMAP,RT_GROUP_ICON,RT_CURSOR,'EMF','WMF'
     {$IFDEF GIF},RT_GIF{$ENDIF}{$IFDEF JPEG},'JPEG'{$ENDIF});

  DateSeparatorFix: Char = '.';
  TimeSeparatorFix: Char = ':';
  ShortDateFormatFix = 'dd/mm/yyyy';
  ShortTimeFormatFix = 'hh:nn:ss';
  DecimalSeparatorFix: Char  = '.';

var
  RHandles: TList;

//------   dates
//        . 
// ONLY RUSSIAN !!!
function AdaptedDateStr(Date: TDateTime): string;

//------   string
//      
// Delete and insert substring in string
function Stuff(S: string; Index,Count: Integer; Source: string): string;

//  \n, \r, \t  \NNN  
// Convert \n, \r, \t and \NNN constructions into chars
function ConvertCodes(Source: string): string;

//           
// Convert symbols from one symbols set to other symbols set
function ConvertSymbols(S, Source, Target: string): string;

//     S
// Number of delimiters in string S
function GetDelimCount(S: String; Delims: TCharSet): Integer;

//     S 
// Position of character in string S from right
function PosR(C: Char; S: String): Integer;

//       S
// Position of quoted with delimiters substring in string S
function PosWordDelims(Srch, S: String; Delims: TCharSet): Integer;

//     - c 
// Add first word into string-list with delimiters
function AddFirstToList(List, S: String; Delim: String): String;

//  TList    
// Convert TList to string with delimiters
function ListToStr(List: TList; Delim: String): String;

//            
//  Windows
// Convert dates and numbers in strings and vice versa with fix delimiters
function StrToDateFix(const S: string): TDateTime;
function StrToTimeFix(const S: string): TDateTime;
function StrToDateTimeFix(const S: string): TDateTime;
function DateToStrFix(Date: TDateTime): string;
function TimeToStrFix(Time: TDateTime): string;
function DateTimeToStrFix(DateTime: TDateTime): string;
function StrToFloatFix(const S: string): Extended;
function FloatToStrFix(Value: Extended): string;
function FormatFloatFix(const Format: string; Value: Extended): string;

//------   numbers
//     
// Cyclic increment inside range
function IncCycle(var Value: Integer;const Offset, MinValue, MaxValue: Integer): Integer;

//  
// Number sign
function Sign(Value: Extended): Integer;

//   
// Raising to integer power
function RaisePower(Value,Power: Integer): Integer;

//    Extended
// Raising to power
function fPower(Value, Power: Extended): Extended;

// 
// Extended round
function fRound(X: Extended; Precision: Integer): Extended;

// ,    0,    
// Round, but return X if round result is 0
function SoftRound(X: Extended; Precision: Integer): Extended;

//     
// Bring to front form
procedure RestoreWindow(Control: TWinControl);

//  
// Redraw window
procedure RefreshWindow(Control: TWinControl; Rect: Pointer);

//------   resources
//     RHandles
// Search resource in RHandles array
function FindResourceInModules(var Module: THandle; const ResName, ResType: PChar): HRSRC;

//   
// Correct name of resource
procedure CorrectResName(var ResName: string);

//    
// Load image from resource
function RLoadPicture(RHandle: THandle; ResName: string; ResKind: TBitmapResKind): THANDLE;

//        
// Load image from resource with converting in bitmap
procedure LoadBitmapFromResource(Bitmap: TBitmap; RHandle: THandle;
                                 ResName: string; ResKind: TBitmapResKind);

//  Bitmap'  ImageList  
// Load bitmap to ImageList from resource
function LoadToImage(Handle: THandle; ResName: PChar; ResKind: TBitmapResKind; Image: TImageList): Boolean;

//------   registry
//  
// Read value
function RegReadValue(Reg: TRegistry; Name: string; Default: string {$IFDEF POLARIS_D4}= ''{$ENDIF}): string;
{$IFDEF POLARIS_D4}overload;
function RegReadValue(Reg: TRegistry; Name: string; Default: Integer = 0): Integer; overload;
function RegReadValue(Reg: TRegistry; Name: string; Default: Extended = 0): Extended; overload;
function RegReadValue(Reg: TRegistry; Name: string; Default: TDateTime = 0): TDateTime; overload;
{$ENDIF}

//------    other
//   Windows
// Get Windows user name
function GetOSUser: string;

//   Variant
// Compare two Variants
function CompareVar(Var1, Var2: Variant): Integer;

implementation

var
  oldSep1, oldSep2: Char;
  oldFormat1, oldFormat2: String;

function AdaptedDateStr(Date: TDateTime): string;
var
  DMY: TrDate;
  newFmt,strMon: string;
begin
  with DMY do begin
    {     }
    DecodeDate(Date,Y,M,D);
    strMon := AnsiLowerCase(LongMonthNames[M]);
    if strMon[Length(strMon)] in ['',''] then
      strMon := Stuff(strMon,Length(strMon),1,'')
    else strMon := strMon+'';
    {   }
    newFmt := LongDateFormat;
    repeat
      D := Pos('MMMM',UpperCase(newFmt));
      if D = 0 then Break;
      newFmt := Stuff(newFmt,D,4,'"'+strMon+'"');
    until D=0;
  end;
  Result := FormatDateTime(newFmt,Date)
end;

function Stuff(S: string; Index,Count: Integer; Source: string): string;
begin
  Result := S;
  Delete(Result,Index,Count);
  Insert(Source,Result,Index);
end;

function ConvertCodes(Source: string): string;
var
  i,j,l: Integer;
begin
  i := 0;
  l := Length(Source);
  Result := '';
  while i < l do begin
    j := Pos('\',Copy(Source,i+1,l))+i;
    if j=i then begin
      Result := Result+Copy(Source,i+1,l);
      break;
    end else begin
      Result := Result+Copy(Source,i+1,j-i-1);
      case Source[j+1] of
        '\': begin
               Result := Result+'\';
               i := j+1;
             end;
        'n': begin
               Result := Result+#13#10;
               i := j+1;
             end;
        'r': begin
               Result := Result+#13;
               i := j+1;
             end;
        't': begin
               Result := Result+#9;
               i := j+1;
             end;
        '0'..'9': begin
                    i := j;
                    while (j < l) and (Source[j+1] in ['0'..'9']) do Inc(j);
                    Result := Result+Chr(StrToInt(Copy(Source,i+1,j-i)));
                    i := j;
                  end
        else begin
          Result := Result+'\';
          i := j;
        end;
      end;
    end;
  end;
end;

function ConvertSymbols(S, Source, Target: string): string;
var
  i, j: Integer;
begin
  Result := '';
  for i:=1 to Length(S) do begin
    j := Pos(S[i], Source);
    if j <> 0 then Result := Result + Target[j]
    else Result := Result + S[i];
  end
end;

function GetDelimCount(S: String; Delims: TCharSet): Integer;
var
  I: Integer;
begin
  Result := 0;
  if Length(S) > 0 then
    for I := 1 to Length(S) do
      if S[I] in Delims then Inc(Result);
end;

function PosR(C: Char; S: String): Integer;
var
  I: Integer;
begin
  Result := 0;
  if Length(S) > 0 then
    for I := Length(S) downto 1 do
      if S[I] = C then begin
        Result := I;
        exit;
      end;
end;

function PosWordDelims(Srch, S: String; Delims: TCharSet): Integer;
var
  i, n: Integer;
begin
  Result := 0;
  n := Length(Srch);
  i := 1;
  repeat
    i := NPos(Srch, S, i);
    if i = 0 then Break;
    if ((i = 1) or (S[i-1] in Delims)) and
       ((i = Length(S)-n+1) or (S[i+n] in Delims)) then  begin
      Result := i;
      Break;
    end;
  until False;
end;

function AddFirstToList(List, S: String; Delim: String): String;
begin
  if Trim(List) = '' then Result := S
  else Result := S + Delim + List;
end;

function ListToStr(List: TList; Delim: String): String;
var
  j: Integer;
begin
  Result := '';
  for j := 0 to List.Count - 1 do begin
    Result := Result + IntToStr(Integer(List.Items[j]));
    if j < List.Count - 1 then Result := Result + Delim;
  end;
end;

function StrToDateFix(const S: string): TDateTime;
begin
  oldSep1 := DateSeparator;
  oldFormat1 := ShortDateFormat;
  DateSeparator := DateSeparatorFix;
  ShortDateFormat := ShortDateFormatFix;
  Result := StrToDate(S);
  DateSeparator := oldSep1;
  ShortDateFormat := oldFormat1;
end;

function StrToTimeFix(const S: string): TDateTime;
begin
  oldSep1 := TimeSeparator;
  oldFormat1 := ShortTimeFormat;
  TimeSeparator := TimeSeparatorFix;
  ShortTimeFormat := ShortTimeFormatFix;
  Result := StrToTime(S);
  TimeSeparator := oldSep1;
  ShortTimeFormat := oldFormat1;
end;

function StrToDateTimeFix(const S: string): TDateTime;
begin
  oldSep1 := DateSeparator;
  oldFormat1 := ShortDateFormat;
  oldSep2 := TimeSeparator;
  oldFormat2 := ShortTimeFormat;
  DateSeparator := DateSeparatorFix;
  ShortDateFormat := ShortDateFormatFix;
  TimeSeparator := TimeSeparatorFix;
  ShortTimeFormat := ShortTimeFormatFix;
  Result := StrToDateTime(S);
  DateSeparator := oldSep1;
  ShortDateFormat := oldFormat1;
  TimeSeparator := oldSep2;
  ShortTimeFormat := oldFormat2;
end;

function DateToStrFix(Date: TDateTime): string;
begin
  oldSep1 := DateSeparator;
  oldFormat1 := ShortDateFormat;
  DateSeparator := DateSeparatorFix;
  ShortDateFormat := ShortDateFormatFix;
  Result := DateToStr(Date);
  DateSeparator := oldSep1;
  ShortDateFormat := oldFormat1;
end;

function TimeToStrFix(Time: TDateTime): string;
begin
  oldSep1 := TimeSeparator;
  oldFormat1 := ShortTimeFormat;
  TimeSeparator := TimeSeparatorFix;
  ShortTimeFormat := ShortTimeFormatFix;
  Result := TimeToStr(Time);
  TimeSeparator := oldSep1;
  ShortTimeFormat := oldFormat1;
end;

function DateTimeToStrFix(DateTime: TDateTime): string;
begin
  oldSep1 := DateSeparator;
  oldFormat1 := ShortDateFormat;
  oldSep2 := TimeSeparator;
  oldFormat2 := ShortTimeFormat;
  DateSeparator := DateSeparatorFix;
  ShortDateFormat := ShortDateFormatFix;
  TimeSeparator := TimeSeparatorFix;
  ShortTimeFormat := ShortTimeFormatFix;
  Result := DateTimeToStr(DateTime);
  DateSeparator := oldSep1;
  ShortDateFormat := oldFormat1;
  TimeSeparator := oldSep2;
  ShortTimeFormat := oldFormat2;
end;

function StrToFloatFix(const S: string): Extended;
begin
  oldSep1 := DecimalSeparator;
  DecimalSeparator  := DecimalSeparatorFix;
  Result := StrToFloat(S);
  DecimalSeparator  := oldSep1;
end;

function FloatToStrFix(Value: Extended): string;
begin
  oldSep1 := DecimalSeparator;
  DecimalSeparator  := DecimalSeparatorFix;
  Result := FloatToStr(Value);
  DecimalSeparator  := oldSep1;
end;

function FormatFloatFix(const Format: string; Value: Extended): string;
begin
  oldSep1 := DecimalSeparator;
  DecimalSeparator  := DecimalSeparatorFix;
  Result := FormatFloat(Format,Value);
  DecimalSeparator  := oldSep1;
end;

function IncCycle(var Value: Integer;const Offset, MinValue, MaxValue: Integer): Integer;
var
  Offs: Integer;
begin
  Result := 0;
  Offs := Offset;
  repeat
    Inc(Value,Offs);
    if Value > MaxValue then begin
      Inc(Result);
      Offs := Value-MaxValue;
      Value := MinValue-1;
    end else
      if Value < MinValue then begin
        Dec(Result);
        Offs := Value-MinValue;
        Value := MaxValue+1;
      end;
  until (Value>=MinValue) and (Value<=MaxValue);
end;

function Sign;
begin
  if Value = 0 then Result := 0 else
    if Value < 0 then Result := -1 else Result := 1;
end;

function RaisePower(Value,Power: Integer): Integer;
var i: Integer;
begin
  Result := 1;
  for i := 1 to Power do Result := Result*Value;
end;

function fPower(Value, Power: Extended): Extended;
begin
  Result := exp(Power*ln(Value));
end;

function fRound(X: Extended; Precision: Integer): Extended;
begin
  Result := Int(X*fPower(10,Precision)+Sign(X)*0.500000001)*fPower(10,-Precision);
end;

function SoftRound(X: Extended; Precision: Integer): Extended;
begin
  Result := fRound(X, Precision);
  if Result = 0 then Result := X;
end;

procedure RestoreWindow(Control: TWinControl);
var
  Action: Integer;
begin
  if Assigned(Control) then begin
    Action := SW_SHOW;
    if IsZoomed(Control.Handle) then Action := SW_MAXIMIZE
    else if IsIconic(Control.Handle) then Action := SW_RESTORE;
    ShowWindowAsync(Control.Handle,Action);
    BringWindowToTop(Control.Handle);
  end;
end;

procedure RefreshWindow(Control: TWinControl; Rect: Pointer);
begin
  if Assigned(Control)
  then RedrawWindow(Control.Handle, Rect, 0,
          RDW_ERASENOW or RDW_UPDATENOW or RDW_FRAME or RDW_ALLCHILDREN or RDW_INVALIDATE);
end;

function FindResourceInModules(var Module: THandle; const ResName, ResType: PChar): HRSRC;
var
  i: Integer;
begin
  Result := FindResource(Module, ResName, ResType);
  if Result > 0 then exit;
  for i:=0 to RHandles.Count-1 do begin
    Result := FindResource(THandle(RHandles.Items[i]), ResName, ResType);
    if Result > 0 then begin
      Module := THandle(RHandles.Items[i]);
      exit;
    end;
  end;
end;

procedure CorrectResName(var ResName: string);
var
  ResID: Integer;
begin
  ResName := Trim(ResName);
  ResID := StrToIntDef(ResName,0);
  if ResID > 0 then ResName := '#'+ResName;
end;

function RLoadPicture(RHandle: THandle; ResName: string; ResKind: TBitmapResKind): THANDLE;
begin
  CorrectResName(ResName);
  if FindResourceInModules(RHandle,PChar(ResName),ResKindNames[ResKind]) = 0 then
    raise EResNotFound.CreateFmt(SResNotFound,[ResName]);
  if ResName[1] = '#' then
    ResName := String(MakeIntResource(Copy(ResName,2,Length(ResName))));
  case ResKind of
    rkBitmap: Result := LoadBitmap(RHandle,PChar(ResName));
    rkIcon:   Result := LoadIcon(RHandle,PChar(ResName));
    rkCursor: Result := LoadCursor(RHandle,PChar(ResName));
  else
    Result := 0;
  end;
end;

procedure LoadBitmapFromResource(Bitmap: TBitmap; RHandle: THandle;
                                 ResName: string; ResKind: TBitmapResKind);
type
  TCG = class of TGraphic;
var
  CG: TCG;
  Image: TGraphic;
  Stream: TResourceStream;
begin
  CorrectResName(ResName);
  if FindResourceInModules(RHandle,PChar(ResName),ResKindNames[ResKind]) = 0 then
    raise EResNotFound.CreateFmt(SResNotFound,[ResName]);
  case ResKind of
    rkBitmap: Bitmap.LoadFromResourceName(RHandle,ResName);
    {rkIcon,}rkEMF,rkWMF{$IFDEF GIF},rkGIF{$ENDIF}{$IFDEF JPEG},rkJPEG{$ENDIF}:
      begin
        Stream := TResourceStream.Create(RHandle,ResName,ResKindNames[ResKind]);
        try
          case ResKind of
//            rkIcon:      CG := TIcon;
            rkEMF,rkWMF: CG := TMetafile;
            {$IFDEF GIF}rkGIF: CG := TGIFImage;{$ENDIF}
            {$IFDEF JPEG}rkJPEG: CG := TJPEGImage;{$ENDIF}
          else
            CG := nil;
          end;
          Image := CG.Create;
          try
            Image.LoadFromStream(Stream);
            if ResKind = rkEMF then TMetafile(Image).Enhanced := True;
            with Image do begin
              Bitmap.Width := Width;
              Bitmap.Height := Height;
            end;
            Bitmap.Canvas.Draw(0,0,Image);
          finally
            Image.Free;
          end;
        finally
          Stream.Free;
        end;
      end;
   rkIcon:
      begin
        Image := TIcon.Create;
        with Image as TIcon do
          try
            Handle := RLoadPicture(RHandle,ResName,rkIcon);
            if not Empty then
              with Bitmap do begin
                Width  := 32;
                Height := 32;
                Canvas.Brush.Color := clFuchsia;
                Canvas.FillRect(Canvas.ClipRect);
                Canvas.Draw(0,0,Image);
              end;
          finally
            Free;
          end;
      end;
  end;
end;

function LoadToImage(Handle: THandle; ResName: PChar; ResKind: TBitmapResKind; Image: TImageList): Boolean;
var
  Bmp: TBitmap;
begin
  Result := False;
  if Assigned(Image)
  then begin
    Bmp := TBitmap.Create;
    try
      LoadBitmapFromResource(Bmp,Handle,ResName,ResKind);
      Image.AddMasked(Bmp,Bmp.TransparentColor);
      Result := True;
    finally
      Bmp.Free;
    end
  end
end;

function RegReadValue(Reg: TRegistry; Name: string; Default: string): string;
{$IFDEF POLARIS_D4}overload;{$ENDIF}
begin
  if Reg.ValueExists(Name) then Result := Reg.ReadString(Name)
  else Result := Default;
end;
{$IFDEF POLARIS_D4}
function RegReadValue(Reg: TRegistry; Name: string; Default: Integer): Integer; overload;
begin
  if Reg.ValueExists(Name) then Result := Reg.ReadInteger(Name)
  else Result := Default;
end;
function RegReadValue(Reg: TRegistry; Name: string; Default: Extended): Extended; overload;
begin
  if Reg.ValueExists(Name) then Result := Reg.ReadFloat(Name)
  else Result := Default;
end;
function RegReadValue(Reg: TRegistry; Name: string; Default: TDateTime): TDateTime; overload;
begin
  if Reg.ValueExists(Name) then Result := Reg.ReadDateTime(Name)
  else Result := Default;
end;
{$ENDIF}

function GetOSUser;
var
  sbuf: PAnsiChar;
  i: DWORD;
  n: Integer;
begin
  n := 80;
  i := n;
  GetMem(sbuf,n);
  GetUserName(sbuf,i);
  Result := sbuf;
  FreeMem(sbuf,n);
end;

function CompareVar(Var1, Var2: Variant): Integer;
begin
  if VarIsEmpty(Var1) then
    if VarIsEmpty(Var2) then Result := 0 else Result := -1
  else
  if VarIsNull(Var1) then
    if VarIsEmpty(Var2) then Result := 1
    else if VarIsNull(Var2) then Result := 0 else Result := -1
  else
  if VarIsEmpty(Var2) or VarIsNull(Var2) then Result := 1 else Result := Var1 - Var2;
end;

initialization
  RHandles := TList.Create;
  RHandles.Add(Pointer(HInstance));
{$IFDEF POLARIS_D5}
  RHandles.Add(Pointer(GetModuleHandle('vcl50.bpl')));
{$ELSE}
{$IFDEF POLARIS_D4}
  RHandles.Add(Pointer(GetModuleHandle('vcl40.bpl')));
{$ELSE}
  RHandles.Add(Pointer(GetModuleHandle('vcl30.bpl')));
{$ENDIF}
{$ENDIF}
finalization
  RHandles.Free;
end.
